@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,8 @@
 #ifndef _CUDA_API_H
 #define _CUDA_API_H 1
 
+#include "defs.h"
+#include "common/common-defs.h"
 #include "cudadebugger.h"
 
 typedef enum {
