@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "common-defs.h"
+#include "common/common-defs.h"
 
 #include "cuda-context.h"
 #include "cuda-options.h"
@@ -149,7 +149,6 @@ void
 contexts_delete (contexts_t ctx)
 {
   context_t  context;
-  list_elt_t next;
   uint32_t   ctxtid;
   uint32_t   tid;
 
@@ -346,7 +345,6 @@ contexts_find_context_by_id  (contexts_t ctx, uint64_t context_id)
 context_t
 contexts_get_active_context (contexts_t ctx, uint32_t tid)
 {
-  uint32_t    ctxtid;
   list_elt_t *stack;
 
   gdb_assert (ctx);
