@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2015-2016 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2015-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -230,7 +230,7 @@ cuda_convenience_get_present_blocks_kernels(struct value **kernel_id_array_value
 
 
 static inline void
-cv_set_uint32_var (char *name, uint32_t val)
+cv_set_uint32_var (const char *name, uint32_t val)
 {
   struct gdbarch *gdbarch = get_current_arch ();
   struct type *type = builtin_type (gdbarch)->builtin_uint32;
@@ -240,7 +240,7 @@ cv_set_uint32_var (char *name, uint32_t val)
 }
 
 static inline void
-cv_set_uint64_var (char *name, uint64_t val)
+cv_set_uint64_var (const char *name, uint64_t val)
 {
   struct gdbarch *gdbarch = get_current_arch ();
   struct type *type = builtin_type (gdbarch)->builtin_uint64;
@@ -444,10 +444,10 @@ static void cv_update_total_kernels_var(void)
 }
 
 struct cv_variable_group {
-  char *group_name;
+  const char *group_name;
   bool enabled;
   void (*update_func)(void);
-  char *group_desc;
+  const char *group_desc;
 };
 
 static struct cv_variable_group cv_var_groups[] =
