@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -20,8 +20,7 @@
 #include "inferior.h"
 #include "language.h"
 #include "target.h"
-#include "common-defs.h"
-#include "string.h"
+#include "common/common-defs.h"
 #include "ui-out.h"
 
 #include "cuda-options.h"
@@ -947,7 +946,6 @@ cuda_print_message_focus (bool switching)
   cuda_coords_t current;
   const uint32_t size = 1000;
   char *string;
-  struct cleanup *chain;
   struct ui_out *uiout = current_uiout;
 
   gdb_assert (cuda_focus_is_device ());
@@ -956,18 +954,18 @@ cuda_print_message_focus (bool switching)
   cuda_coords_get_current (&current);
   cuda_coords_to_fancy_string (&current, string, size);
 
-  if (ui_out_is_mi_like_p (uiout))
+  if (uiout->is_mi_like_p ())
     {
-      chain = make_cleanup_ui_out_tuple_begin_end (uiout, "CudaFocus");
-      ui_out_field_int (uiout, "device"    , current.dev);
-      ui_out_field_int (uiout, "sm"        , current.sm);
-      ui_out_field_int (uiout, "warp"      , current.wp);
-      ui_out_field_int (uiout, "lane"      , current.ln);
-      ui_out_field_int (uiout, "kernel"    , current.kernelId);
-      ui_out_field_int (uiout, "grid"      , current.gridId);
-      ui_out_field_fmt (uiout, "blockIdx"  , "(%u,%u,%u)", current.blockIdx.x, current.blockIdx.y, current.blockIdx.z);
-      ui_out_field_fmt (uiout, "threadIdx" , "(%u,%u,%u)", current.threadIdx.x, current.threadIdx.y, current.threadIdx.z);
-      do_cleanups (chain);
+      ui_out_emit_type<ui_out_type_tuple> cuda_focus(uiout, "CudaFocus");
+
+      uiout->field_int ("device"    , current.dev);
+      uiout->field_int ("sm"        , current.sm);
+      uiout->field_int ("warp"      , current.wp);
+      uiout->field_int ("lane"      , current.ln);
+      uiout->field_int ("kernel"    , current.kernelId);
+      uiout->field_int ("grid"      , current.gridId);
+      uiout->field_fmt ("blockIdx"  , "(%u,%u,%u)", current.blockIdx.x, current.blockIdx.y, current.blockIdx.z);
+      uiout->field_fmt ("threadIdx" , "(%u,%u,%u)", current.threadIdx.x, current.threadIdx.y, current.threadIdx.z);
     }
   else
     {
