@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2015-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2015-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -15,12 +15,15 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <string.h>
-
 #include "defs.h"
-#include "common-defs.h"
+#include "common/common-defs.h"
 #include "ui-out.h"
-#include "gdb_signals.h"
+#include "common/gdb_signals.h"
+
+#ifdef CUDA_DEBUG_LINE_EXTENSION
+#include "symfile.h"
+#include "demangle.h"
+#endif
 
 #include "cuda-coords.h"
 #include "cuda-exceptions.h"
@@ -102,32 +105,59 @@ print_exception_origin (cuda_exception_t exception)
       return;
     }
 
-  ui_out_text (uiout, "The exception was triggered at ");
+  uiout->text ("The exception was triggered at ");
   pc = warp_get_error_pc (c.dev, c.sm, c.wp);
+
+#ifdef CUDA_DEBUG_LINE_EXTENSION
+  struct cuda_debug_inline_info *inline_info = NULL;
+
+  struct obj_section *section = find_pc_overlay (pc);
+  sal = find_pc_sect_line ((CORE_ADDR)pc, section, 0, &inline_info);
+#else
   sal = find_pc_line ((CORE_ADDR)pc, 0);
+#endif
 
   if (sal.symtab && sal.line)
     {
-      filename = strrchr (sal.symtab->filename, '/');
+      filename = strrchr ((char *)sal.symtab->filename, '/');
       if (filename)
         ++filename;
       else
         filename = (char *) sal.symtab->filename;
     }
 
-  ui_out_text (uiout, "PC ");
-  ui_out_field_fmt (uiout, "pc", "0x%llx",  (unsigned long long)pc);
+  uiout->text ("PC ");
+  uiout->field_fmt ("pc", "0x%llx",  (unsigned long long)pc);
 
   if (filename)
     {
-      ui_out_text         (uiout, " (");
-      ui_out_field_string (uiout, "filename"    , filename);
-      ui_out_text         (uiout, ":");
-      ui_out_field_int    (uiout, "line"        , sal.line);
-      ui_out_text         (uiout, ")");
+      uiout->text         (" (");
+      uiout->field_string ("filename"    , filename);
+      uiout->text         (":");
+      uiout->field_int    ("line"        , sal.line);
+
+#ifdef CUDA_DEBUG_LINE_EXTENSION
+      if (inline_info)
+	{
+	  char *demangled = language_demangle (language_def (current_language->la_language),
+					       inline_info->function,
+					       DMGL_ANSI);
+
+	  const char *inline_function = demangled ? demangled : inline_info->function;
+	  uiout->text         (" in ");
+	  uiout->field_string ("inline_function", inline_function);
+	  xfree (demangled);
+
+	  uiout->text         (" inlined from ");
+	  uiout->field_string ("inline_filename", lbasename (inline_info->filename));
+	  uiout->text         (":");
+	  uiout->field_int    ("inline_line", inline_info->line);
+	}
+#endif
+      uiout->text         (")");
     }
 
-  ui_out_text (uiout, "\n");
+  uiout->text ("\n");
 }
 
 static void
@@ -139,9 +169,9 @@ print_exception_device (cuda_exception_t exception)
 
   c = cuda_exception_get_coords (exception);
 
-  ui_out_text (current_uiout, "The exception was triggered in device ");
-  ui_out_field_int    (current_uiout, "device", c.dev);
-  ui_out_text         (current_uiout, ".\n");
+  current_uiout->text ("The exception was triggered in device ");
+  current_uiout->field_int ("device", c.dev);
+  current_uiout->text (".\n");
 }
 
 static void
@@ -178,21 +208,21 @@ print_lane_illegal_address_message (cuda_exception_t exception)
         break;
     };
 
-  ui_out_text (uiout, "\nIllegal access to address ");
+  uiout->text ("\nIllegal access to address ");
   if (addr_space)
     {
-      ui_out_text (uiout, "(");
-      ui_out_field_string (uiout, "error-address-space", addr_space);
-      ui_out_text (uiout, ")");
+      uiout->text ("(");
+      uiout->field_string ("error-address-space", addr_space);
+      uiout->text (")");
     }
-  ui_out_field_fmt (uiout, "address", "0x%llx",  (unsigned long long)address);
-  ui_out_text (uiout, " detected.\n");
+  uiout->field_fmt ("address", "0x%llx",  (unsigned long long)address);
+  uiout->text (" detected.\n");
 }
 
 static void
 print_assert_message (cuda_exception_t exception)
 {
-  ui_out_text (current_uiout, "\nAssertion failed.\n");
+  current_uiout->text ("\nAssertion failed.\n");
 }
 
 static void
@@ -206,9 +236,9 @@ print_exception_name (cuda_exception_t exception)
   value = (enum gdb_signal) cuda_exception_get_value (exception);
   name = gdb_signal_to_string (value);
 
-  ui_out_text (current_uiout, "\nCUDA Exception: ");
-  ui_out_field_string (current_uiout, "exception_name", name);
-  ui_out_text (current_uiout, "\n");
+  current_uiout->text ("\nCUDA Exception: ");
+  current_uiout->field_string ("exception_name", name);
+  current_uiout->text ("\n");
 }
 
 void
@@ -259,8 +289,6 @@ cuda_exception_hit_p (cuda_exception_t exception)
   cuda_coords_t c = CUDA_INVALID_COORDS, filter = CUDA_WILDCARD_COORDS;
   cuda_api_warpmask tmp;
   cuda_iterator itr;
-  uint64_t error_pc;
-  bool error_pc_available;
 
   /* Iteration should be limited to single sm if sstep is active */
   if (cuda_sstep_is_active())
