@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "environ.h"
 #include "inferior.h"
 #include "gdbcmd.h"
+#include "remote.h"
 
 #include "cuda-options.h"
 #include "cuda-state.h"
@@ -44,14 +45,14 @@ struct cmd_list_element *showdebugcudalist;
  * cuda prefix
  */
 static void
-set_cuda (char *arg, int from_tty)
+set_cuda (const char *arg, int from_tty)
 {
   printf_unfiltered (_("\"set cuda\" must be followed by the name of a cuda subcommand.\n"));
   help_list (setcudalist, "set cuda ", (enum command_class) -1, gdb_stdout);
 }
 
 static void
-show_cuda (char *args, int from_tty)
+show_cuda (const char *args, int from_tty)
 {
   cmd_show_list (showcudalist, from_tty, "");
 }
@@ -72,14 +73,14 @@ cuda_options_initialize_cuda_prefix (void)
  * set debug cuda
  */
 static void
-set_debug_cuda (char *arg, int from_tty)
+set_debug_cuda (const char *arg, int from_tty)
 {
   printf_unfiltered (_("\"set debug cuda\" must be followed by the name of a debug cuda subcommand.\n"));
   help_list (setdebugcudalist, "set debug cuda ", (enum command_class) -1, gdb_stdout);
 }
 
 static void
-show_debug_cuda (char *args, int from_tty)
+show_debug_cuda (const char *args, int from_tty)
 {
   cmd_show_list (showdebugcudalist, from_tty, "");
 }
@@ -147,12 +148,12 @@ cuda_show_debug_trace (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_debug_trace (char *args, int from_tty,
-                        struct cmd_list_element *c)
+cuda_set_debug_trace (const char *args, int from_tty,
+		      struct cmd_list_element *c)
 {
   char *flags = NULL;
   char *ptr,*flg;
-  int cnt, rc;
+  int cnt;
 
   cuda_debug_trace_flags = 0;
 
@@ -196,7 +197,7 @@ out:
     xfree (flags);
 
   if (cuda_remote)
-    cuda_remote_set_option ();
+    cuda_remote_set_option (get_current_remote_target ());
 }
 
 static void
@@ -254,11 +255,11 @@ cuda_show_debug_notifications (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_debug_notifications (char *args, int from_tty,
+cuda_set_debug_notifications (const char *args, int from_tty,
                               struct cmd_list_element *c)
 {
   if (cuda_remote)
-    cuda_remote_set_option ();
+    cuda_remote_set_option (get_current_remote_target ());
 }
 
 static void
@@ -294,11 +295,11 @@ cuda_show_debug_libcudbg (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_debug_libcudbg (char *args, int from_tty,
+cuda_set_debug_libcudbg (const char *args, int from_tty,
                          struct cmd_list_element *c)
 {
   if (cuda_remote)
-    cuda_remote_set_option ();
+    cuda_remote_set_option (get_current_remote_target ());
 }
 
 static void
@@ -334,12 +335,12 @@ cuda_show_debug_convenience_vars (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_debug_convenience_vars (char *args, int from_tty,
+cuda_set_debug_convenience_vars (const char *args, int from_tty,
                                  struct cmd_list_element *c)
 {
   char *groups = NULL;
   char *ptr,*grp;
-  int cnt, rc;
+  int rc;
 
   if (cuda_debug_convenience_vars)
     groups = xstrdup (cuda_debug_convenience_vars);
@@ -446,11 +447,11 @@ cuda_options_initialize_memcheck (void)
 bool
 cuda_options_memcheck (void)
 {
-  struct gdb_environ *env = current_inferior ()->environment;
+  struct gdb_environ &env = current_inferior ()->environment;
 
   /* Memcheck auto value is determined by the CUDA_MEMCHECK env, which
    * can only be evaluated when the inferior is running. */
-  cuda_memcheck_auto = env && !!get_in_environ (env, "CUDA_MEMCHECK");
+  cuda_memcheck_auto = !!env.get ("CUDA_MEMCHECK");
 
   return cuda_memcheck == AUTO_BOOLEAN_TRUE ||
         (cuda_memcheck == AUTO_BOOLEAN_AUTO && cuda_memcheck_auto);
@@ -463,7 +464,7 @@ int cuda_coalescing_auto;
 enum auto_boolean cuda_coalescing;
 
 static void
-cuda_set_coalescing (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_coalescing (const char *args, int from_tty, struct cmd_list_element *c)
 {
   printf_filtered ("Coalescing of the CUDA commands output is %s.\n",
                    cuda_options_coalescing () ? "on" : "off");
@@ -520,10 +521,10 @@ cuda_show_notify (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_notify (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_notify (const char *args, int from_tty, struct cmd_list_element *c)
 {
   if (cuda_remote)
-    cuda_remote_set_option ();
+    cuda_remote_set_option (get_current_remote_target ());
 }
 
 static void
@@ -580,7 +581,7 @@ cuda_show_break_on_launch (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_break_on_launch (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_break_on_launch (const char *args, int from_tty, struct cmd_list_element *c)
 {
   if (cuda_options_auto_breakpoints_needed ())
     cuda_auto_breakpoints_add_locations ();
@@ -649,7 +650,7 @@ cuda_show_disassemble_from (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_disassemble_from (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_disassemble_from (const char *args, int from_tty, struct cmd_list_element *c)
 {
   cuda_system_flush_disasm_cache ();
 }
@@ -683,13 +684,68 @@ cuda_options_disassemble_from_elf_image (void)
   return cuda_disassemble_from == cuda_disassemble_from_elf_image;
 }
 
+/*
+ * set cuda disassemble_per
+ */
+const char  cuda_disassemble_per_function[] = "function";
+const char  cuda_disassemble_per_file [] = "file";
+
+const char *cuda_disassemble_per_enums[] = {
+  cuda_disassemble_per_function,
+  cuda_disassemble_per_file,
+  NULL
+};
+
+static const char *cuda_disassemble_per;
+
+static void
+cuda_show_disassemble_per (struct ui_file *file, int from_tty,
+                          struct cmd_list_element *c, const char *value)
+{
+  printf_filtered ("CUDA code is disassembled per %s.\n", value);
+}
+
+static void
+cuda_set_disassemble_per (const char *args, int from_tty, struct cmd_list_element *c)
+{
+}
+
+static void
+cuda_options_initialize_disassemble_per (void)
+{
+  cuda_disassemble_per = cuda_disassemble_per_function;
+
+  add_setshow_enum_cmd ("disassemble_per", class_cuda,
+                        cuda_disassemble_per_enums, &cuda_disassemble_per,
+                        _("Choose whether to disassemble a function at a time (fast) "
+                          "or file at a time (slow)."),
+                        _("Show if disassembly is per-function or per-file."),
+                        _("Choose how disassembly is performed:\n"
+                          "  function : done on a per-function basis (fast))\n"
+                          "  file     : done on a per-file basis (slower at first, then faster)\n"),
+                        cuda_set_disassemble_per, cuda_show_disassemble_per,
+                        &setcudalist, &showcudalist);
+}
+
+bool
+cuda_options_disassemble_per_function (void)
+{
+  return cuda_disassemble_per == cuda_disassemble_per_function;
+}
+
+bool
+cuda_options_disassemble_per_file (void)
+{
+  return cuda_disassemble_per == cuda_disassemble_per_file;
+}
+
 /*
  * set cuda hide_internal_frames
  */
 int cuda_hide_internal_frames;
 
 static void
-cuda_set_hide_internal_frames (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_hide_internal_frames (const char *args, int from_tty, struct cmd_list_element *c)
 {
   // force rebuilding frame stack to see the change
   reinit_frame_cache ();
@@ -759,7 +815,7 @@ cuda_options_force_set_launch_notification_update (void)
 }
 
 static void
-cuda_set_show_kernel_events (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_show_kernel_events (const char *args, int from_tty, struct cmd_list_element *c)
 {
   if (cuda_options_auto_breakpoints_needed ())
     cuda_auto_breakpoints_add_locations ();
@@ -874,7 +930,7 @@ cuda_options_show_context_events (void)
 int cuda_launch_blocking;
 
 static void
-cuda_set_launch_blocking (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_launch_blocking (const char *args, int from_tty, struct cmd_list_element *c)
 {
   if (cuda_launch_blocking)
       printf_filtered ("On the next run, the CUDA kernel launches will be blocking.\n");
@@ -959,9 +1015,9 @@ cuda_options_thread_selection_physical (void)
 }
 
 static void
-show_cuda_copyright_command (char *ignore, int from_tty)
+show_cuda_copyright_command (const char *ignore, int from_tty)
 {
-  print_gdb_version (gdb_stdout);
+  print_gdb_version (gdb_stdout, false);
   printf_filtered ("\n");
 }
 
@@ -1001,7 +1057,7 @@ cuda_show_api_failures (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_api_failures  (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_api_failures  (const char *args, int from_tty, struct cmd_list_element *c)
 {
   cuda_update_report_driver_api_error_flags ();
 }
@@ -1080,8 +1136,8 @@ cuda_options_initialize_software_preemption (void)
 bool
 cuda_options_software_preemption (void)
 {
-  struct gdb_environ *env = current_inferior ()->environment;
-  char *cuda_dsp = env ? get_in_environ (env, "CUDA_DEBUGGER_SOFTWARE_PREEMPTION") : NULL;
+  struct gdb_environ &env = current_inferior ()->environment;
+  const char *cuda_dsp = env.get ("CUDA_DEBUGGER_SOFTWARE_PREEMPTION");
 
   /* Software preemption auto value is determined by the
      CUDA_DEBUGGER_SOFTWARE_PREEMPTION env var */
@@ -1159,11 +1215,10 @@ cuda_options_variable_value_cache_enabled (void)
 }
 
 static void
-cuda_print_statistics (char *args, int from_tty)
+cuda_print_statistics (const char *args, int from_tty)
 {
   uint32_t cnt;
   const CUDBGIPCStat_t *st;
-  struct cleanup *table_cleanup, *row_cleanup;
   struct ui_out *uiout = current_uiout;
 
   /* column headers */
@@ -1172,8 +1227,9 @@ cuda_print_statistics (char *args, int from_tty)
   const char *header_avg = "Average call time(usec)";
   const char *header_min = "Min call time(usec)";
   const char *header_max = "Max call time(usec)";
+  const char *header_total = "Total call time(usec)";
 
-  int name_width = strlen(header_name);
+  size_t name_width = strlen(header_name);
   int row_no = 0;
   double total = 0;
 
@@ -1183,40 +1239,39 @@ cuda_print_statistics (char *args, int from_tty)
       if (!st) continue;
 
       total += st->total_time;
-      name_width = max(name_width,strlen(st->name));
+      name_width = std::max(name_width, strlen(st->name));
       row_no++;
     }
 
-  table_cleanup = make_cleanup_ui_out_table_begin_end (uiout, 5, row_no, "CUDBGAPIStatTable");
-  ui_out_table_header (uiout, name_width,           ui_left,   "name",  header_name);
-  ui_out_table_header (uiout, strlen(header_calls), ui_center, "calls", header_calls);
-  ui_out_table_header (uiout, strlen(header_avg),   ui_center, "avg",   header_avg);
-  ui_out_table_header (uiout, strlen(header_min),   ui_center, "min",   header_min);
-  ui_out_table_header (uiout, strlen(header_max),   ui_center, "max",   header_max);
-  ui_out_table_body (uiout);
+  ui_out_emit_table table_cleanup (uiout, 6, row_no, "CUDBGAPIStatTable");
+  uiout->table_header (name_width,           ui_left,  "name",  header_name);
+  uiout->table_header (strlen(header_calls), ui_right, "calls", header_calls);
+  uiout->table_header (strlen(header_avg),   ui_right, "avg",   header_avg);
+  uiout->table_header (strlen(header_min),   ui_right, "min",   header_min);
+  uiout->table_header (strlen(header_max),   ui_right, "max",   header_max);
+  uiout->table_header (strlen(header_total), ui_right, "total", header_total);
+  uiout->table_body ();
 
   for (cnt=0; cnt < CUDBGIPC_API_STAT_MAX; cnt++)
     {
       st = cudbgipcGetProfileStat (cnt);
       if (!st) continue;
 
-      row_cleanup = make_cleanup_ui_out_tuple_begin_end (uiout, "CUDBGAPIStatRow");
-      ui_out_field_string (uiout, "name", st->name);
-      ui_out_field_int (uiout, "calls", st->times_called);
-      ui_out_field_int (uiout, "avg", (int)(st->total_time/st->times_called));
-      ui_out_field_int (uiout, "min", (int)st->min_time);
-      ui_out_field_int (uiout, "max", (int)st->max_time);
-      ui_out_text (uiout, "\n");
-      do_cleanups (row_cleanup);
+      ui_out_emit_tuple row_cleanup (uiout, "CUDBGAPIStatRow");
+      uiout->field_string ("name", st->name);
+      uiout->field_int ("calls", st->times_called);
+      uiout->field_int ("avg", (int)(st->total_time/st->times_called));
+      uiout->field_int ("min", (int)st->min_time);
+      uiout->field_int ("max", (int)st->max_time);
+      uiout->field_int ("total", (int)st->total_time);
+      uiout->text ("\n");
     }
 
-  do_cleanups (table_cleanup);
-
   printf_unfiltered ("Total time spend in CUDBG API is %f sec\n", total*1e-6);
 }
 
 
-static int cuda_gpu_collect_stats = 0;
+static int cuda_gpu_collect_stats = 1;
 
 static void
 cuda_show_cuda_gpu_collect_stats (struct ui_file *file, int from_tty,
@@ -1266,12 +1321,9 @@ cuda_options_value_extrapolation_enabled (void)
 }
 
 static void
-cuda_print_regmap (char *args, int from_tty)
+cuda_print_regmap (const char *args, int from_tty)
 {
-
-  struct objfile *objfile;
-
-  ALL_OBJFILES (objfile)
+  for (objfile *objfile : current_program_space->objfiles ())
     {
       if (!objfile->cuda_objfile)
         continue;
@@ -1347,7 +1399,7 @@ cuda_show_stop_signal (struct ui_file *file, int from_tty,
 }
 
 static void
-cuda_set_stop_signal (char *args, int from_tty, struct cmd_list_element *c)
+cuda_set_stop_signal (const char *args, int from_tty, struct cmd_list_element *c)
 {
   if (strcasecmp(cuda_stop_signal_string,"SIGURG")==0)
      cuda_stop_signal = GDB_SIGNAL_URG;
@@ -1355,7 +1407,7 @@ cuda_set_stop_signal (char *args, int from_tty, struct cmd_list_element *c)
      cuda_stop_signal = GDB_SIGNAL_TRAP;
 
   if (cuda_remote)
-    cuda_remote_set_option ();
+    cuda_remote_set_option (get_current_remote_target ());
 }
 
 unsigned cuda_options_stop_signal (void)
@@ -1422,6 +1474,7 @@ cuda_options_initialize (void)
   cuda_options_initialize_break_on_launch ();
   cuda_options_initialize_api_failures ();
   cuda_options_initialize_disassemble_from ();
+  cuda_options_initialize_disassemble_per ();
   cuda_options_initialize_hide_internal_frames ();
   cuda_options_initialize_show_kernel_events ();
   cuda_options_initialize_show_context_events ();
