@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -51,6 +51,9 @@
  * be valid.
  */
 
+#ifndef _CUDA_PARSER_H
+#define _CUDA_PARSER_H 1
+
 #include "cudadebugger.h"
 #include "defs.h"
 #include "cuda-tdep.h"
@@ -118,3 +121,5 @@ extern command_t start_token;        /* Used internally by the parser */
 void cuda_parser (const char * input, command_t command, cuda_parser_result_t **result,
                   cuda_coords_special_value_t dflt_value);
 void cuda_parser_print (cuda_parser_result_t *result);
+
+#endif
