@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2013 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2013-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,7 @@
 #ifdef __QNXHOST__
 # include "remote-nto.h"
 #endif /* __QNXHOST__ */
-#include "rsp-low.h"
+#include "common/rsp-low.h"
 
 #define TEXTURE_DIM_MAX 4
 
