@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2013-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2013-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -94,9 +94,9 @@ int
 cuda_gdb_get_tid (ptid_t ptid)
 {
   if (cuda_platform_supports_tid ())
-    return ptid_get_lwp (ptid);
+    return ptid.lwp ();
   else
-    return ptid_get_pid (ptid);
+    return ptid.pid ();
 }
 
 /*---------------------------------------- sigpending -------------------------------------*/
@@ -233,7 +233,7 @@ cuda_check_pending_sigint (ptid_t ptid)
 /*---------------------------------------- Routines ---------------------------------------*/
 
 ATTRIBUTE_PRINTF(1, 2) void
-cuda_trace (char *fmt, ...)
+cuda_trace (const char *fmt, ...)
 {
   struct cuda_trace_msg *msg;
   va_list ap;
@@ -347,7 +347,7 @@ cuda_get_symbol_cache_size (void)
 }
 
 CORE_ADDR
-cuda_get_symbol_address_from_cache (char* name)
+cuda_get_symbol_address_from_cache (const char* name)
 {
   int i;
 
