@@ -1,16 +1,16 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2017 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -20,15 +20,15 @@
 #include <cuda-tdep-server.h>
 #else
 #include <defs.h>
-#include <common-defs.h>
+#include <common/common-defs.h>
 #include <cuda-options.h>
 #include <cuda-tdep.h>
-#include <ptid.h>
+#include <common/ptid.h>
 #include <inferior.h>
 #if __QNXTARGET__
 # include "remote-nto.h"
 # define PUTPKT_BINARY qnx_putpkt_binary
-# define GETPKT qnx_getpkt
+# define GETPKT qnx_getpkt_sane
 #else
 # define PUTPKT_BINARY putpkt_binary
 # define GETPKT getpkt_sane
@@ -37,7 +37,6 @@
 
 #include <signal.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
@@ -46,7 +45,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <pthread.h>
-#include "rsp-low.h"
+#include "common/rsp-low.h"
 
 #include <cuda-utils.h>
 #include <libcudbg.h>
@@ -54,7 +53,7 @@
 #include <cudadebugger.h>
 
 /*Forward declarations */
-static ATTRIBUTE_PRINTF(1, 2) void cudbgipc_trace(char *fmt, ...);
+static ATTRIBUTE_PRINTF(1, 2) void cudbgipc_trace(const char *fmt, ...);
 
 /*Globals */
 CUDBGNotifyNewEventCallback cudbgDebugClientCallback = NULL;
@@ -134,12 +133,12 @@ cudbgipcCreate(CUDBGIPC_t *ipc, int from, int to, int flags)
     /* If cuda-gdb is launched as root, make pipes are writeable by UID of debugged process */
 #ifndef GDBSERVER
     if (getuid() == 0) {
-         int pid = (int) ptid_get_pid (inferior_ptid);
-         if (pid > 0 && !cuda_gdb_chown_to_pid_uid (pid, ipc->name)) {
-             cudbgipc_trace("Changing ownership to pid %d uid failed for %s, errno=%d",
+      int pid = (int) inferior_ptid.pid ();
+      if (pid > 0 && !cuda_gdb_chown_to_pid_uid (pid, ipc->name)) {
+	cudbgipc_trace("Changing ownership to pid %d uid failed for %s, errno=%d",
                        pid, ipc->name, errno);
-             return CUDBG_ERROR_COMMUNICATION_FAILURE;
-         }
+	return CUDBG_ERROR_COMMUNICATION_FAILURE;
+      }
     }
 #endif
 
@@ -294,9 +293,9 @@ cudbgipcInitializeCommCB(void)
 
 static char *outBuffer = NULL;
 static char *inBuffer = NULL;
-static long inBufferSize = 0;
-static uint32_t outBufferSize = 0;
-static uint32_t outBufferUsed = 0;
+static size_t inBufferSize = 0;
+static size_t outBufferSize = 0;
+static size_t outBufferUsed = 0;
 
 static CUDBGResult
 cudbgipcInitializeRemote (void)
@@ -305,29 +304,39 @@ cudbgipcInitializeRemote (void)
 
   inBuffer = (char *) xmalloc (inBufferSize = 65535);
 
-   outBufferUsed = snprintf (outBuffer, outBufferSize, "vCUDA;");
-   return CUDBG_SUCCESS;
+  outBufferUsed = snprintf (outBuffer, outBufferSize, "vCUDA;");
+  return CUDBG_SUCCESS;
 }
 
 static CUDBGResult
-cudbgipcAppendRemote(void *d, uint32_t size)
+cudbgipcAppendRemote(void *d, size_t size)
 {
     int out_len, bytes_processed;
 
+    /* Guard against integer overflow */
+    /* FIXME: gdbserver should handle sizes larger than INT_MAX bytes */
+    if (size > INT_MAX || outBufferUsed > outBufferUsed + size)
+      return CUDBG_ERROR_COMMUNICATION_FAILURE;
+
     if (outBufferUsed + size > outBufferSize)
       {
-	char *buf = (char *)realloc (outBuffer, outBufferSize + size);
+        char *buf = (char *)realloc (outBuffer, outBufferUsed + size);
 
         if (!buf)
-	  return CUDBG_ERROR_UNKNOWN;
+          return CUDBG_ERROR_UNKNOWN;
         outBuffer = buf;
-        outBufferSize += size;
+        outBufferSize = outBufferUsed + size;
       }
 
-    out_len = remote_escape_output ((const gdb_byte *) d, size, 1,
-				    (gdb_byte *) outBuffer+outBufferUsed,
-				    &bytes_processed, 
-				    outBufferSize-outBufferUsed);
+    /* Guard against integer overflow */
+    /* FIXME: gdbserver should handle sizes larger than INT_MAX bytes */
+    if (outBufferSize - outBufferUsed > INT_MAX)
+      return CUDBG_ERROR_COMMUNICATION_FAILURE;
+
+    out_len = remote_escape_output ((const gdb_byte *) d, (int) size, 1,
+				    (gdb_byte *) outBuffer + outBufferUsed,
+				    &bytes_processed,
+				    (int) (outBufferSize - outBufferUsed));
     gdb_assert (bytes_processed == size);
 
     outBufferUsed += out_len;
@@ -337,50 +346,61 @@ cudbgipcAppendRemote(void *d, uint32_t size)
 static CUDBGResult
 cudbgipcRequestRemote(void **d, size_t *size)
 {
-    static char *recvBuffer = NULL;
-    static long recvBufferSize = 0;
+    static gdb::char_vector rcvbuf(get_remote_packet_size ());
     size_t totalRecvSize = 0;
     int recvBytes;
 
-    if (!recvBuffer) {
-        recvBufferSize = get_remote_packet_size();
-        recvBuffer = (char *) xmalloc (recvBufferSize);
-    }
-
     PUTPKT_BINARY (outBuffer, outBufferUsed);
 
-
     do {
-        recvBytes = GETPKT (&recvBuffer, &recvBufferSize, 0);
+        recvBytes = GETPKT (&rcvbuf, 0);
 
         /* Handle errors */
         if (recvBytes < 3)
             return CUDBG_ERROR_COMMUNICATION_FAILURE;
-        if (memcmp (recvBuffer, "OK;", strlen("OK;")) != 0 &&
-            memcmp (recvBuffer, "MP;", strlen("MP;")) != 0) {
-            gdb_assert (recvBuffer[0] == 'E' &&
-                  recvBuffer[1]>='0' && recvBuffer[1]<='9' &&
-                  recvBuffer[2]>='0' && recvBuffer[2]<='9');
+        if (memcmp (rcvbuf.data (), "OK;", strlen("OK;")) != 0 &&
+            memcmp (rcvbuf.data (), "MP;", strlen("MP;")) != 0)
+          {
+            gdb_assert (rcvbuf.data ()[0] == 'E' &&
+                  rcvbuf.data ()[1]>='0' && rcvbuf.data ()[1]<='9' &&
+                  rcvbuf.data ()[2]>='0' && rcvbuf.data ()[2]<='9');
 
             outBufferUsed = snprintf (outBuffer, outBufferSize, "vCUDA;");
-            return (CUDBGResult) atoi(recvBuffer+1);
-        }
+            return (CUDBGResult) atoi(rcvbuf.data ()+1);
+          }
+
+        /* Guard against integer overflow */
+        /* FIXME: gdbserver should handle sizes larger than INT_MAX bytes */
+        if (totalRecvSize > totalRecvSize + recvBytes)
+          return CUDBG_ERROR_COMMUNICATION_FAILURE;
 
         /* Adjust input buffer size, if necessary */
         if (inBufferSize < totalRecvSize + recvBytes)
-	  inBuffer = (char *) xrealloc (inBuffer, inBufferSize += 2*recvBytes);
+          {
+            size_t new_inBufferSize = inBufferSize + 2*recvBytes;
+            /* Guard against integer overflow */
+            if (inBufferSize > new_inBufferSize)
+              return CUDBG_ERROR_COMMUNICATION_FAILURE;
+            inBuffer = (char *) xrealloc (inBuffer, new_inBufferSize);
+            inBufferSize = new_inBufferSize;
+          }
 
-        recvBytes = remote_unescape_input ((const gdb_byte *) (recvBuffer + strlen("OK;")),
+        /* Guard against integer overflow */
+        /* FIXME: gdbserver should handle sizes larger than INT_MAX bytes */
+        if (inBufferSize - totalRecvSize > INT_MAX)
+          return CUDBG_ERROR_COMMUNICATION_FAILURE;
+
+        recvBytes = remote_unescape_input ((const gdb_byte *) (rcvbuf.data () + strlen("OK;")),
 					   recvBytes - strlen ("OK;"),
                                            (gdb_byte *) (inBuffer + totalRecvSize),
-					   inBufferSize - totalRecvSize);
+					   (int) (inBufferSize - totalRecvSize));
         totalRecvSize += recvBytes;
         /* If a multi-packet reply received, send a request for more data */
-        if (memcmp (recvBuffer, "MP;", strlen ("MP;")) == 0) {
+        if (memcmp (rcvbuf.data (), "MP;", strlen ("MP;")) == 0) {
             outBufferUsed = snprintf (outBuffer, outBufferSize, "vCUDARetr;%lu", (unsigned long)totalRecvSize);
             PUTPKT_BINARY (outBuffer, outBufferUsed);
         }
-    } while ( memcmp (recvBuffer, "OK;", strlen ("OK;")) != 0);
+    } while ( memcmp (rcvbuf.data (), "OK;", strlen ("OK;")) != 0);
 
     outBufferUsed = snprintf (outBuffer, outBufferSize, "vCUDA;");
     *d = inBuffer;
@@ -394,8 +414,8 @@ cudbgipcRequestRemote(void **d, size_t *size)
 static CUDBGResult
 cudbgipcPush(CUDBGIPC_t *out)
 {
-    int64_t writeCount = 0;
-    uint64_t offset = 0;
+    ssize_t writeCount = 0;
+    size_t offset = 0;
     char *buf;
 
     gdb_assert (out);
@@ -413,8 +433,8 @@ cudbgipcPush(CUDBGIPC_t *out)
               pthread_kill (cudagdbMainThreadHandle, SIGINT);
 
             if (errno != EAGAIN && errno != EINTR) {
-                cudbgipc_trace("Fifo write error (from=%u, to=%u, out->dataSize=%lu, offset=%lu, errno=%d)",
-                               out->from, out->to, (unsigned long)out->dataSize, (unsigned long)offset, errno);
+                cudbgipc_trace("Fifo write error (from=%u, to=%u, out->dataSize=%zu, offset=%zu, errno=%d)",
+                               out->from, out->to, out->dataSize, offset, errno);
                 return CUDBG_ERROR_COMMUNICATION_FAILURE;
             }
             writeCount = 0;
@@ -427,10 +447,10 @@ cudbgipcPush(CUDBGIPC_t *out)
 }
 
 static CUDBGResult
-cudbgipcRead(CUDBGIPC_t *in, void *buf, uint32_t size)
+cudbgipcRead(CUDBGIPC_t *in, void *buf, size_t size)
 {
-    int64_t readCount = 0;
-    uint64_t offset = 0;
+    ssize_t readCount = 0;
+    size_t offset = 0;
 
     gdb_assert (in);
 
@@ -446,8 +466,8 @@ cudbgipcRead(CUDBGIPC_t *in, void *buf, uint32_t size)
               pthread_kill (cudagdbMainThreadHandle, SIGINT);
 
             if (errno != EAGAIN && errno != EINTR) {
-                cudbgipc_trace("Fifo read error (from=%u, to=%u, size=%u, offset=%lu, errno=%d)",
-                               in->from, in->to, size, (unsigned long)offset, errno);
+                cudbgipc_trace("Fifo read error (from=%u, to=%u, size=%zu, offset=%zu, errno=%d)",
+                               in->from, in->to, size, offset, errno);
                 return CUDBG_ERROR_COMMUNICATION_FAILURE;
             }
             readCount = 0;
@@ -469,6 +489,12 @@ cudbgipcPull(CUDBGIPC_t *in)
         return res;
     }
 
+    /* Guard against incorrect preamble read */
+    if (in->dataSize == 0) {
+        cudbgipc_trace("Read zero sized preamble");
+        return CUDBG_ERROR_COMMUNICATION_FAILURE;
+    }
+
     /* Allocate memory given the size */
     if ((in->data = (char *) realloc(in->data, in->dataSize)) == 0) {
         cudbgipc_trace("Memory reallocation failed (res=%d)", res);
@@ -523,10 +549,10 @@ cudbgipcWait(CUDBGIPC_t *in)
 }
 
 static CUDBGResult
-cudbgipcAppendLocal(void *d, uint32_t size)
+cudbgipcAppendLocal(void *d, size_t size)
 {
     CUDBGResult res;
-    uint32_t dataSize = 0;
+    size_t dataSize = 0;
     void *data = NULL;
 
     if (!commOut.initialized) {
@@ -536,6 +562,10 @@ cudbgipcAppendLocal(void *d, uint32_t size)
     }
 
     dataSize = commOut.dataSize + size;
+    /* Guard against integer overflow */
+    if (commOut.dataSize > dataSize) {
+        return CUDBG_ERROR_COMMUNICATION_FAILURE;
+    }
     if ((data = realloc(commOut.data, dataSize)) == NULL)
         return CUDBG_ERROR_COMMUNICATION_FAILURE;
 
@@ -548,7 +578,7 @@ cudbgipcAppendLocal(void *d, uint32_t size)
 }
 
 CUDBGResult
-cudbgipcAppend(void *d, uint32_t size)
+cudbgipcAppend(void *d, size_t size)
 {
 #ifndef GDBSERVER
     if (cuda_remote)
@@ -591,7 +621,7 @@ cudbgipcRequest(void **d, size_t *size)
 }
 
 CUDBGResult
-cudbgipcCBWaitForData(void *d, uint32_t size)
+cudbgipcCBWaitForData(void *d, size_t size)
 {
     CUDBGResult res;
 
@@ -718,7 +748,7 @@ const CUDBGIPCStat_t *cudbgipcGetProfileStat(uint32_t id)
     return api_call_stat[id].name ? api_call_stat+id : NULL;
 }
 
-ATTRIBUTE_PRINTF(1, 2) void cudbgipc_trace(char *fmt, ...)
+ATTRIBUTE_PRINTF(1, 2) void cudbgipc_trace(const char *fmt, ...)
 {
 #ifdef GDBSERVER
   struct cuda_trace_msg *msg;
@@ -735,12 +765,12 @@ ATTRIBUTE_PRINTF(1, 2) void cudbgipc_trace(char *fmt, ...)
     cuda_first_trace_msg = msg;
   else
     cuda_last_trace_msg->next = msg;
-  sprintf (msg->buf, "[CUDAGDB] libcudbg ipc");
+  sprintf (msg->buf, "[CUDAGDB] libcudbg ipc ");
   vsnprintf (msg->buf + strlen (msg->buf), sizeof (msg->buf), fmt, ap);
   msg->next = NULL;
   cuda_last_trace_msg = msg;
 #else
-  fprintf (stderr, "[CUDAGDB] libcudbg ipc");
+  fprintf (stderr, "[CUDAGDB] libcudbg ipc ");
   vfprintf (stderr, fmt, ap);
   fprintf (stderr, "\n");
   fflush (stderr);
