@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2017 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -43,21 +43,84 @@
 
 #include "top.h"
 
-static struct target_ops *host_target_ops = NULL;
-static struct target_ops  host_remote_target_ops;
-static struct target_ops  host_extended_remote_target_ops;
-#ifdef __QNXTARGET__
-static struct target_ops  host_qnx_target_ops;
-#endif
 static bool sendAck = false;
-static bool interrupt_flag = false;
 #ifdef __QNXTARGET__
 static bool symbols_are_set = false;
 #endif
 
-static void
-cuda_do_resume (struct target_ops *ops, ptid_t ptid,
-                     int sstep, int host_sstep, enum gdb_signal ts)
+template <class parent>
+class cuda_remote_target : public parent
+{
+public:
+  cuda_remote_target ()
+  { }
+
+  virtual void kill () override;
+
+  virtual void mourn_inferior () override;
+
+  virtual void detach (inferior *arg0, int arg1) override;
+
+  virtual void resume (ptid_t arg0,
+		       int TARGET_DEBUG_PRINTER (target_debug_print_step) arg1,
+		       enum gdb_signal arg2) override;
+
+  void cuda_do_resume (ptid_t ptid, int sstep, int host_sstep, enum gdb_signal ts);
+
+
+  virtual ptid_t wait (ptid_t arg0, struct target_waitstatus * arg1,
+		       int TARGET_DEBUG_PRINTER (target_debug_print_options) arg2) override;
+
+  virtual void fetch_registers (struct regcache *arg0, int arg1) override;
+
+  virtual void store_registers (struct regcache *arg0, int arg1) override;
+
+  virtual int insert_breakpoint (struct gdbarch *arg0,
+				 struct bp_target_info *arg1) override;
+
+  virtual int remove_breakpoint (struct gdbarch *arg0,
+				 struct bp_target_info *arg1,
+				 enum remove_bp_reason arg2) override;
+
+  virtual enum target_xfer_status xfer_partial (enum target_object object,
+						const char *annex,
+						gdb_byte *readbuf,
+						const gdb_byte *writebuf,
+						ULONGEST offset, ULONGEST len,
+						ULONGEST *xfered_len) override;
+
+  virtual struct gdbarch *thread_architecture (ptid_t arg0) override;
+
+  virtual void prepare_to_store (struct regcache *arg0) override;
+};
+
+static
+remote_target *
+cuda_new_remote_target (void)
+{
+#ifdef __QNXTARGET__
+  return new cuda_remote_target<qnx_remote_target<remote_target>> ();
+#else
+  return new cuda_remote_target<remote_target> ();
+#endif
+}
+
+#ifndef __QNXTARGET__
+static
+remote_target *
+cuda_new_extended_remote_target (void)
+{
+#ifdef __QNXTARGET__
+  return new cuda_remote_target<qnx_remote_target<extended_remote_target>> ();
+#else
+  return new cuda_remote_target<extended_remote_target> ();
+#endif
+}
+#endif
+
+template <class parent>
+void
+cuda_remote_target<parent>::cuda_do_resume (ptid_t ptid, int sstep, int host_sstep, enum gdb_signal ts)
 {
   uint32_t dev;
 
@@ -72,7 +135,7 @@ cuda_do_resume (struct target_ops *ops, ptid_t ptid,
             device_resume (dev);
 
       // resume the host
-      host_target_ops->to_resume (ops, ptid, sstep, ts);
+      parent::resume (ptid, sstep, ts);
       return;
     }
 
@@ -92,7 +155,7 @@ cuda_do_resume (struct target_ops *ops, ptid_t ptid,
 	     we can report the device single-stepping event back.  */
 #ifndef __QNXTARGET__
 	  /* On QNX this workaround does not seem to be required */
-	  remote_report_event (1);
+	  remote_report_event (this, 1);
 #endif
 	  return;
 	}
@@ -113,13 +176,11 @@ cuda_do_resume (struct target_ops *ops, ptid_t ptid,
         device_resume (dev);
 
   // resume the host
-  host_target_ops->to_resume (ops, ptid, 0, ts);
+  parent::resume (ptid, 0, ts);
 }
 
-int remote_query_attached (int pid);
-
 static void
-cuda_initialize_remote_target (void)
+cuda_initialize_remote_target (struct target_ops *ops)
 {
   CUDBGResult get_debugger_api_res;
   CUDBGResult set_callback_api_res;
@@ -132,6 +193,7 @@ cuda_initialize_remote_target (void)
   bool driver_is_compatible;
   char *dev_type;
   char *sm_type;
+  remote_target *remote = (remote_target *)ops;
 #ifdef __QNXTARGET__
   struct cmd_list_element *alias = NULL;
   struct cmd_list_element *prefix_cmd = NULL;
@@ -154,7 +216,7 @@ cuda_initialize_remote_target (void)
   /* Send the target the symbols it needs */
   if (!symbols_are_set)
     {
-      cuda_remote_set_symbols (&symbols_are_set);
+      cuda_remote_set_symbols ((struct remote_target *)ops, &symbols_are_set);
       if (!symbols_are_set)
         {
           return;
@@ -163,7 +225,8 @@ cuda_initialize_remote_target (void)
 #endif
 
   /* Ask cuda-gdbserver to initialize. */
-  cuda_remote_initialize (&get_debugger_api_res, &set_callback_api_res, &api_initialize_res,
+  cuda_remote_initialize (remote,
+			  &get_debugger_api_res, &set_callback_api_res, &api_initialize_res,
                           &cuda_initialized, &cuda_debugging_enabled, &driver_is_compatible);
 
   cuda_api_handle_get_api_error (get_debugger_api_res);
@@ -181,12 +244,13 @@ cuda_initialize_remote_target (void)
   cuda_system_initialize ();
   for (dev_id = 0; dev_id < cuda_system_get_num_devices (); dev_id++)
     {
-      cuda_remote_query_device_spec (dev_id, &num_sms, &num_warps, &num_lanes,
+      cuda_remote_query_device_spec (remote,
+				     dev_id, &num_sms, &num_warps, &num_lanes,
                                      &num_registers, &dev_type, &sm_type);
       cuda_system_set_device_spec (dev_id, num_sms, num_warps, num_lanes,
                                    num_registers, dev_type, sm_type);
     }
-  cuda_remote_set_option ();
+  cuda_remote_set_option (remote);
   cuda_gdb_session_create ();
   cuda_update_report_driver_api_error_flags ();
   cuda_initialize_driver_api_error_report ();
@@ -201,59 +265,21 @@ cuda_finalize_remote_target (void)
 }
 #endif
 
-static void
-cuda_remote_open (const char *name, int from_tty)
+template <class parent>
+void
+cuda_remote_target<parent>::kill ()
 {
-  host_target_ops = &host_remote_target_ops;
-  host_target_ops->to_open (name, from_tty);
-  /* CUDA - remote attach only when all host threads are enumerated */
-  cuda_remote_attach();
-}
-
-static void
-cuda_extended_remote_open (const char *name, int from_tty)
-{
-  host_target_ops = &host_extended_remote_target_ops;
-  host_target_ops->to_open (name, from_tty);
-  /* CUDA - remote attach only when all host threads are enumerated */
-  cuda_remote_attach();
-}
-
-#ifdef __QNXTARGET__
-static void
-cuda_qnx_open (const char *name, int from_tty)
-{
-  host_target_ops = &host_qnx_target_ops;
-  host_target_ops->to_open (name, from_tty);
-  /* CUDA - remote attach only when all host threads are enumerated */
-  cuda_remote_attach();
-}
-#endif
-
-static void
-cuda_remote_close (struct target_ops *ops)
-{
-  gdb_assert (host_target_ops);
-
-  host_target_ops->to_close (ops);
-}
-
-static void
-cuda_remote_kill (struct target_ops *ops)
-{
-  gdb_assert (host_target_ops);
-
   cuda_api_finalize ();
   cuda_cleanup ();
   cuda_gdb_session_destroy ();
-  host_target_ops->to_kill (ops);
+
+  parent::kill ();
 }
 
-static void
-cuda_remote_mourn_inferior (struct target_ops *ops)
+template <class parent>
+void
+cuda_remote_target<parent>::mourn_inferior (void)
 {
-  gdb_assert (host_target_ops);
-
   /* Mark breakpoints uninserted in case something tries to delete a
      breakpoint while we delete the inferior's threads (which would
      fail, since the inferior is long gone).  */
@@ -263,23 +289,21 @@ cuda_remote_mourn_inferior (struct target_ops *ops)
   {
     cuda_cleanup ();
     cuda_gdb_session_destroy ();
-    host_target_ops->to_mourn_inferior (ops);
+
+    parent::mourn_inferior ();
   }
 }
 
 extern int cuda_host_want_singlestep;
 
-static void
-cuda_remote_resume (struct target_ops *ops,
-                    ptid_t ptid, int sstep, enum gdb_signal ts)
+template <class parent>
+void
+cuda_remote_target<parent>::resume (ptid_t ptid, int sstep, enum gdb_signal ts)
 {
-  uint32_t dev;
-  cuda_coords_t c;
   bool cuda_event_found = false;
   CUDBGEvent event;
   int host_want_sstep = cuda_host_want_singlestep;
 
-  gdb_assert (host_target_ops);
   cuda_trace ("cuda_resume: sstep=%d", sstep);
   cuda_host_want_singlestep = 0;
 
@@ -339,7 +363,7 @@ cuda_remote_resume (struct target_ops *ops,
       sendAck = false;
     }
 
-  cuda_do_resume (ops, ptid, sstep, host_want_sstep, ts);
+  cuda_do_resume (ptid, sstep, host_want_sstep, ts);
 
   cuda_clock_increment ();
 
@@ -349,9 +373,9 @@ cuda_remote_resume (struct target_ops *ops,
 extern void (*gdb_old_sighand_func) (int);
 extern void remote_interrupt (int);
 
-static ptid_t
-cuda_remote_wait (struct target_ops *ops,
-                  ptid_t ptid, struct target_waitstatus *ws, int target_options)
+template <class parent>
+ptid_t
+cuda_remote_target<parent>::wait (ptid_t ptid, struct target_waitstatus *ws, int target_options)
 {
   ptid_t r;
   uint32_t dev, dev_id;
@@ -362,7 +386,6 @@ cuda_remote_wait (struct target_ops *ops,
   struct thread_info *tp;
   cuda_coords_t c;
 
-  gdb_assert (host_target_ops);
   cuda_trace ("cuda_wait");
 
   if (cuda_exception_is_valid (cuda_exception))
@@ -382,9 +405,9 @@ cuda_remote_wait (struct target_ops *ops,
       /* Check if C-c was sent to a remote application or if quit_flag is set.
          quit_flag is set by gdb handle_sigint() signal handler */
 #ifdef __QNXTARGET__
-      if (cuda_remote_check_pending_sigint (r) || check_quit_flag())
+      if (cuda_remote_check_pending_sigint (this, r) || check_quit_flag())
 #else /* __QNXTARGET__ */
-      if (cuda_remote_check_pending_sigint () || check_quit_flag())
+      if (cuda_remote_check_pending_sigint (this) || check_quit_flag())
 #endif /* __QNXTARGET__ */
         {
           ws->kind = TARGET_WAITKIND_STOPPED;
@@ -438,7 +461,7 @@ cuda_remote_wait (struct target_ops *ops,
   else {
     cuda_trace ("cuda_wait: host_wait\n");
     cuda_coords_invalidate_current ();
-    r = host_target_ops->to_wait (ops, ptid, ws, target_options);
+    r = parent::wait (ptid, ws, target_options);
 
     /* GDB reads events asynchronously without blocking. The remote may have
        taken too long to reply and GDB did not get any events back.  Check if
@@ -456,14 +479,14 @@ cuda_remote_wait (struct target_ops *ops,
   }
 
   if (!cuda_initialized)
-    cuda_initialize_remote_target ();
+    cuda_initialize_remote_target (current_top_target ());
 
   /* Suspend all the CUDA devices. */
   cuda_trace ("cuda_wait: suspend devices");
   for (dev = 0; dev < cuda_system_get_num_devices (); ++dev)
     device_suspend (dev);
 
-  cuda_remote_query_trace_message ();
+  cuda_remote_query_trace_message (this);
   /* Check for ansynchronous events.  These events do not require
      acknowledgement to the debug API, and may arrive at any time
      without an explicit notification. */
@@ -527,7 +550,7 @@ cuda_remote_wait (struct target_ops *ops,
     {
       /* Finished attaching to the CUDA app.
          Preferably switch focus to a device if possible */
-      struct inferior *inf = find_inferior_pid (ptid_get_pid (r));
+      struct inferior *inf = find_inferior_pid (r.pid ());
       cuda_trace ("cuda_wait: stopped because we attached to the CUDA app");
       cuda_api_set_attach_state (CUDA_ATTACH_STATE_COMPLETE);
       inf->control.stop_soon = STOP_QUIETLY;
@@ -536,7 +559,7 @@ cuda_remote_wait (struct target_ops *ops,
   else if (cuda_api_get_attach_state () == CUDA_ATTACH_STATE_DETACH_COMPLETE)
     {
       /* Finished detaching from the CUDA app. */
-      struct inferior *inf = find_inferior_pid (ptid_get_pid (r));
+      struct inferior *inf = find_inferior_pid (r.pid ());
       cuda_trace ("cuda_wait: stopped because we detached from the CUDA app");
       inf->control.stop_soon = STOP_QUIETLY;
     }
@@ -580,11 +603,11 @@ cuda_remote_wait (struct target_ops *ops,
       LONGEST offset = arch_ptr_size == 8 ? 0x10 : 0x0c;
       LONGEST read = 0;
       gdb_byte *buf = (gdb_byte *)&addr;
-      int inf_exec = is_executing (inferior_ptid);
+      int inf_exec = find_thread_ptid (inferior_ptid)->executing;
 
       /* Mark inferior_ptid as not executing while reading object signal info*/
       set_executing (inferior_ptid, 0);
-      read = target_read (host_target_ops, TARGET_OBJECT_SIGNAL_INFO, NULL, buf, offset, len);
+      read = target_read (this, TARGET_OBJECT_SIGNAL_INFO, NULL, buf, offset, len);
       set_executing (inferior_ptid, inf_exec);
 
       /* Check the results */
@@ -606,24 +629,20 @@ cuda_remote_wait (struct target_ops *ops,
   return r;
 }
 
-static void
-cuda_remote_fetch_registers (struct target_ops *ops,
-                             struct regcache *regcache,
-                             int regno)
+template <class parent>
+void
+cuda_remote_target<parent>::fetch_registers (struct regcache *regcache, int regno)
 {
   uint64_t val = 0;
   cuda_coords_t c;
   struct gdbarch *gdbarch = get_regcache_arch (regcache);
   uint32_t pc_regnum = gdbarch_pc_regnum (gdbarch);
-  int num_regs = gdbarch_num_regs (gdbarch);
   enum register_status status;
 
-  gdb_assert (host_target_ops);
-
   /* delegate to the host routines when not on the device */
   if (!cuda_focus_is_device ())
     {
-      host_target_ops->to_fetch_registers (ops, regcache, regno);
+      parent::fetch_registers (regcache, regno);
       return;
     }
 
@@ -633,9 +652,9 @@ cuda_remote_fetch_registers (struct target_ops *ops,
      device PC */
   if (regno == -1)
     {
-      host_target_ops->to_fetch_registers (ops, regcache, regno);
+      parent::fetch_registers (regcache, regno);
       val = lane_get_virtual_pc (c.dev, c.sm, c.wp, c.ln);
-      regcache_raw_supply (regcache, pc_regnum, &val);
+      regcache->raw_supply (pc_regnum, &val);
       return;
     }
 
@@ -643,7 +662,7 @@ cuda_remote_fetch_registers (struct target_ops *ops,
   if (regno == pc_regnum )
     {
       val = lane_get_virtual_pc (c.dev, c.sm, c.wp, c.ln);
-      regcache_raw_supply (regcache, pc_regnum, &val);
+      regcache->raw_supply (pc_regnum, &val);
       return;
     }
 
@@ -651,45 +670,44 @@ cuda_remote_fetch_registers (struct target_ops *ops,
     {
       /* raw register */
       val = lane_get_register (c.dev, c.sm, c.wp, c.ln, regno);
-      regcache_raw_supply (regcache, regno, &val);
+      regcache->raw_supply (regno, &val);
       return;
     }
 
   status = cuda_pseudo_register_read (gdbarch, regcache, regno, (gdb_byte *)&val);
   gdb_assert (status == REG_VALID);
   
-  regcache_raw_supply (regcache, regno, &val);
+  regcache->raw_supply (regno, &val);
 }
 
-static void
-cuda_remote_store_registers (struct target_ops *ops,
-                             struct regcache *regcache,
-                             int regno)
+template <class parent>
+void
+cuda_remote_target<parent>::store_registers (struct regcache *regcache, int regno)
 {
   uint64_t val;
   struct gdbarch *gdbarch = get_regcache_arch (regcache);
   uint32_t pc_regnum = gdbarch_pc_regnum (gdbarch);
   int num_regs = gdbarch_num_regs (gdbarch);
 
-  gdb_assert (host_target_ops);
   gdb_assert (regno >= 0 && regno < num_regs);
 
   if (!cuda_focus_is_device ())
     {
-      host_target_ops->to_store_registers (ops, regcache, regno);
+      parent::store_registers (regcache, regno);
       return;
     }
 
   if (regno == pc_regnum)
     error (_("The PC of CUDA thread is not writable"));
 
-  regcache_raw_collect (regcache, regno, &val);
+  regcache->raw_collect (regno, &val);
   cuda_register_write (gdbarch, regcache, regno, (gdb_byte *) &val);
 }
 
-static int
-cuda_remote_insert_breakpoint (struct target_ops *ops, struct gdbarch *gdbarch,
-			       struct bp_target_info *bp_tgt)
+template <class parent>
+int
+cuda_remote_target<parent>::insert_breakpoint (struct gdbarch *gdbarch,
+					       struct bp_target_info *bp_tgt)
 {
   uint32_t dev;
   bool inserted;
@@ -699,7 +717,7 @@ cuda_remote_insert_breakpoint (struct target_ops *ops, struct gdbarch *gdbarch,
               gdbarch_bfd_arch_info (gdbarch)->arch == bfd_arch_aarch64);
 
   if (!bp_tgt->owner || !bp_tgt->owner->cuda_breakpoint)
-    return host_target_ops->to_insert_breakpoint (ops, gdbarch, bp_tgt);
+    return parent::insert_breakpoint (gdbarch, bp_tgt);
 
   /* Insert the breakpoint on whatever device accepts it (valid address). */
   inserted = false;
@@ -715,13 +733,13 @@ cuda_remote_insert_breakpoint (struct target_ops *ops, struct gdbarch *gdbarch,
   return !inserted;
 }
 
-static int
-cuda_remote_remove_breakpoint (struct target_ops *ops, struct gdbarch *gdbarch,
-			       struct bp_target_info *bp_tgt,
-			       enum remove_bp_reason reason)
+template <class parent>
+int
+cuda_remote_target<parent>::remove_breakpoint (struct gdbarch *gdbarch,
+					       struct bp_target_info *bp_tgt,
+					       enum remove_bp_reason reason)
 {
   uint32_t dev;
-  CORE_ADDR cuda_addr;
   bool removed;
 
   gdb_assert (bp_tgt->owner != NULL ||
@@ -729,7 +747,7 @@ cuda_remote_remove_breakpoint (struct target_ops *ops, struct gdbarch *gdbarch,
               gdbarch_bfd_arch_info (gdbarch)->arch == bfd_arch_aarch64);
 
   if (!bp_tgt->owner || !bp_tgt->owner->cuda_breakpoint)
-    return host_target_ops->to_remove_breakpoint (ops, gdbarch, bp_tgt, reason);
+    return parent::remove_breakpoint (gdbarch, bp_tgt, reason);
 
   /* Removed the breakpoint on whatever device accepts it (valid address). */
   removed = false;
@@ -745,8 +763,8 @@ cuda_remote_remove_breakpoint (struct target_ops *ops, struct gdbarch *gdbarch,
    only the si_signo matters. We do not save the siginfo object. Instead we
    save only the signo. Therefore any read/write to any other field of the
    siginfo object will have no effect or will return 0. */
-static enum target_xfer_status
-cuda_remote_xfer_siginfo (struct target_ops *ops, enum target_object object,
+enum target_xfer_status
+cuda_remote_xfer_siginfo (remote_target *remote, enum target_object object,
                           const char *annex, gdb_byte *readbuf,
                           const gdb_byte *writebuf, ULONGEST offset,
 			  LONGEST len, ULONGEST *xfered_len)
@@ -759,7 +777,7 @@ cuda_remote_xfer_siginfo (struct target_ops *ops, enum target_object object,
   gdb_byte buf[sizeof (siginfo_t) + 24];
   siginfo_t *siginfo = (siginfo_t *) buf;
 
-  gdb_assert (host_target_ops);
+  gdb_assert (remote);
   gdb_assert (object == TARGET_OBJECT_SIGNAL_INFO);
   gdb_assert (readbuf || writebuf);
 
@@ -789,22 +807,21 @@ cuda_remote_xfer_siginfo (struct target_ops *ops, enum target_object object,
   return TARGET_XFER_OK;
 }
 
-static enum target_xfer_status
-cuda_remote_xfer_partial (struct target_ops *ops,
-                          enum target_object object, const char *annex,
-                          gdb_byte *readbuf, const gdb_byte *writebuf,
-                          ULONGEST offset, ULONGEST len, ULONGEST *xfered_len)
+template <class parent>
+enum target_xfer_status
+cuda_remote_target<parent>::xfer_partial (enum target_object object, const char *annex,
+					  gdb_byte *readbuf, const gdb_byte *writebuf,
+					  ULONGEST offset, ULONGEST len, ULONGEST *xfered_len)
 {
   enum target_xfer_status status = TARGET_XFER_E_IO;
   uint32_t dev, sm, wp, ln;
 
-  gdb_assert (host_target_ops);
   /* If focus set on device, call the host routines directly */
   if (!cuda_focus_is_device ())
     {
-      status = host_target_ops->to_xfer_partial (ops, object, annex, readbuf,
-                                                 writebuf, offset, len,
-						 xfered_len);
+      status = parent::xfer_partial (object, annex, readbuf,
+				     writebuf, offset, len,
+				     xfered_len);
       return status;
     }
 
@@ -840,22 +857,24 @@ cuda_remote_xfer_partial (struct target_ops *ops,
     /* When stopping on the device, build a simple siginfo object */
     case TARGET_OBJECT_SIGNAL_INFO:
 
-      status = cuda_remote_xfer_siginfo (ops, object, annex, readbuf,
-					      writebuf, offset, len,
-					      xfered_len);
+      status = cuda_remote_xfer_siginfo (this, object, annex, readbuf,
+					 writebuf, offset, len,
+					 xfered_len);
       break;
   }
 
   if (*xfered_len < len)
-    status = host_target_ops->to_xfer_partial (ops, object, annex, readbuf,
-                                               writebuf, offset, len,
-					       xfered_len);
-
+    {
+      status = parent::xfer_partial (object, annex, readbuf,
+				     writebuf, offset, len,
+				     xfered_len);
+    }
   return status;
 }
 
-static struct gdbarch *
-cuda_remote_thread_architecture (struct target_ops *ops, ptid_t ptid)
+template <class parent>
+struct gdbarch *
+cuda_remote_target<parent>::thread_architecture (ptid_t ptid)
 {
   if (cuda_focus_is_device ())
     return cuda_get_gdbarch ();
@@ -863,13 +882,13 @@ cuda_remote_thread_architecture (struct target_ops *ops, ptid_t ptid)
     return target_gdbarch();
 }
 
-static void
-cuda_remote_prepare_to_store (struct target_ops *ops, struct regcache *regcache)
+template <class parent>
+void
+cuda_remote_target<parent>::prepare_to_store (struct regcache *regcache)
 {
-  gdb_assert (host_target_ops);
   if (get_regcache_arch (regcache) == cuda_get_gdbarch ())
     return;
-  host_target_ops->to_prepare_to_store (ops, regcache);
+  parent::prepare_to_store (regcache);
 }
 
 void
@@ -884,26 +903,32 @@ cuda_remote_attach (void)
   struct cmd_list_element *alias = NULL;
   struct cmd_list_element *prefix_cmd = NULL;
   struct cmd_list_element *cmd = NULL;
-  char *cudbgApiInitForAttach = "cudbgApiInit(1)";
+  const char *cudbgApiAttach = "(void) cudbgApiAttach()";
   CORE_ADDR debugFlagAddr;
   CORE_ADDR sessionIdAddr;
   CORE_ADDR attachDataAvailableFlagAddr;
   const unsigned char one = 1;
   unsigned char attachDataAvailable;
   uint32_t sessionId = 0;
-  unsigned int timeOut = 5000000; /* 5 seconds */
+  unsigned int timeOut = 5000; // ms
   unsigned int timeElapsed = 0;
-  const unsigned int sleepTime = 1000;
+  const unsigned int sleepTime = 1; // ms
+  bool need_retry = 0;
+  uint64_t internal_error_code = 0;
+  unsigned retry_count = 0;
+  unsigned retry_delay = 100; // ms
+  unsigned app_init_timeout = 5000; // ms
+  struct cleanup *cleanup = NULL;
 
   if (cuda_api_get_attach_state() != CUDA_ATTACH_STATE_NOT_STARTED && 
     cuda_api_get_attach_state() != CUDA_ATTACH_STATE_DETACH_COMPLETE)
     return;
 #ifndef __QNXTARGET__
-  if (!remote_query_attached (inferior_ptid.pid))
+  if (!get_current_remote_target ()->remote_query_attached (inferior_ptid.pid ()))
     return;
 #endif
 
-  cuda_initialize_remote_target();
+  cuda_initialize_remote_target (current_top_target ());
 
   debugFlagAddr = cuda_get_symbol_address (_STRING_(CUDBG_IPC_FLAG_NAME));
   sessionIdAddr = cuda_get_symbol_address (_STRING_(CUDBG_SESSION_ID));
@@ -928,20 +953,62 @@ cuda_remote_attach (void)
   if (!lookup_cmd_composition ("call", &alias, &prefix_cmd, &cmd))
     error (_("Failed to initiate attach."));
 
-  /* Fork off the CUDA debugger process from the inferior */
-  cmd_func (cmd, cudbgApiInitForAttach, 0);
-
   attachDataAvailableFlagAddr = cuda_get_symbol_address (_STRING_(CUDBG_ATTACH_HANDLER_AVAILABLE));
 
   /* If this is not available, the CUDA driver doesn't support attaching.  */
   if (!attachDataAvailableFlagAddr)
     error (_("This CUDA driver does not support attaching to a running CUDA process."));
 
+  cuda_sigtrap_set_silent ();
+
+  do
+  {
+    /* Try to init debugger's backend */
+    cleanup = cuda_gdb_bypass_signals ();
+    cmd_func (cmd, cudbgApiAttach, 0);
+    do_cleanups (cleanup);
+
+    internal_error_code = cuda_get_last_driver_internal_error_code();
+
+    /* CUDBG_ERROR_ATTACH_NOT_POSSIBLE can be returned in two scenarios:
+     * 1. Attach is really not possible
+     * 2. Critical section's mutex is taken, attaching would cause a deadlock */
+    need_retry = (unsigned int)internal_error_code == CUDBG_ERROR_ATTACH_NOT_POSSIBLE;
+
+    if (need_retry)
+    {
+      /* Resume the target */
+      prepare_execution_command (current_top_target (), true);
+      continue_1 (true);
+
+      usleep(retry_delay * 1000);
+
+      /* Trigger the future wait() */
+      kill (current_inferior ()->pid, SIGTRAP);
+
+      /* Get control back */
+      wait_for_inferior ();
+      set_running (minus_one_ptid, 0);
+
+      retry_count++;
+    }
+  } while (need_retry && (retry_count * retry_delay < app_init_timeout));
+
+  /* We are re-using the ATTACH_NOT_POSSIBLE error code for delayed attach,
+   * therefore a timeout will allow us to determine if the error code is
+   * genuinely not possible. */
+  if (need_retry)
+    error (_("Attaching not possible. "
+             "Please verify that software preemption is disabled "
+             "and that nvidia-cuda-mps-server is not running."));
+
+  cuda_sigtrap_restore_settings ();
+
   /* Wait till the backend has started up and is ready to service API calls */
   while (cuda_api_initialize () != CUDBG_SUCCESS)
     {
       if (timeElapsed < timeOut)
-        usleep(sleepTime);
+        usleep(sleepTime * 1000);
       else
         error (_("Timed out waiting for the CUDA API to initialize."));
 
@@ -960,14 +1027,14 @@ cuda_remote_attach (void)
 
       for (cnt=0;
 	   cnt < 1000
-           && cuda_api_get_attach_state () == CUDA_ATTACH_STATE_IN_PROGRESS;
+	     && cuda_api_get_attach_state () == CUDA_ATTACH_STATE_IN_PROGRESS;
            cnt++)
-	  {
-	      prepare_execution_command (&current_target, true);
-	      continue_1 (false);
-	      wait_for_inferior ();
-	      normal_stop ();
-	  }
+	{
+	  prepare_execution_command (current_top_target (), true);
+	  continue_1 (false);
+	  wait_for_inferior ();
+	  normal_stop ();
+	}
 
       /* All threads are stopped at this point.  */
       set_running (minus_one_ptid, 0);
@@ -982,26 +1049,26 @@ cuda_remote_attach (void)
     }
 }
 
-static void
-cuda_remote_detach(struct target_ops *ops, const char *args, int from_tty)
+template <class parent>
+void
+cuda_remote_target<parent>::detach (inferior *inf, int from_tty)
 {
-  gdb_assert (host_target_ops);
-
   /* If attach wasn't completed,
      treat the inferior as a host-only process */
   if (cuda_api_get_attach_state() == CUDA_ATTACH_STATE_COMPLETE &&
-     remote_query_attached (inferior_ptid.pid))
+      get_current_remote_target ()->remote_query_attached (inferior_ptid.pid ()))
     cuda_do_detach (true);
-  host_target_ops->to_detach (host_target_ops, args, from_tty);
 
+  parent::detach (inf, from_tty);
 }
 
 void
 #ifdef __QNXTARGET__
 cuda_version_handshake (const char *version_string)
 #else /* __QNXTARGET__ */
-cuda_remote_version_handshake (const struct protocol_feature *feature,
-                               enum packet_support support,
+cuda_remote_version_handshake (remote_target *remote,
+			       const struct protocol_feature *feature,
+			       enum packet_support support,
                                const char *version_string)
 #endif /* __QNXTARGET__ */
 {
@@ -1030,47 +1097,3 @@ cuda_remote_version_handshake (const struct protocol_feature *feature,
            CUDBG_API_VERSION_REVISION,
            server_major, server_minor, server_rev);
 }
-
-void
-cuda_remote_add_target (struct target_ops *t)
-{
-  /* Save the original set of target operations */
-  if (strcmp (t->to_shortname, "remote") == 0)
-    {
-      host_remote_target_ops = *t;
-      t->to_open = cuda_remote_open;
-    }
-  else if (strcmp (t->to_shortname, "extended-remote") == 0)
-    {
-      host_extended_remote_target_ops = *t;
-      t->to_open = cuda_extended_remote_open;
-    }
-#ifdef __QNXTARGET__
-  else if (strcmp (t->to_shortname, "qnx") == 0)
-    {
-      host_qnx_target_ops = *t;
-      t->to_open = cuda_qnx_open;
-    }
-#endif
-  else
-    gdb_assert (0);
-
-  alloc_cuda_packet_buffer ();
-  make_final_cleanup (free_cuda_packet_buffer, NULL);
-
-  /* Override what we need to */
-  t->to_close			= cuda_remote_close;
-  t->to_kill			= cuda_remote_kill;
-  t->to_mourn_inferior		= cuda_remote_mourn_inferior;
-  t->to_detach			= cuda_remote_detach;
-  t->to_resume			= cuda_remote_resume;
-  t->to_wait			= cuda_remote_wait;
-  t->to_fetch_registers		= cuda_remote_fetch_registers;
-  t->to_store_registers         = cuda_remote_store_registers;
-  t->to_insert_breakpoint       = cuda_remote_insert_breakpoint;
-  t->to_remove_breakpoint       = cuda_remote_remove_breakpoint;
-  t->to_xfer_partial            = cuda_remote_xfer_partial;
-  t->to_thread_architecture	= cuda_remote_thread_architecture;
-  t->to_prepare_to_store        = cuda_remote_prepare_to_store;
-}
-
